from importlib.util import find_spec
from pathlib import Path

from django.utils.translation import gettext_lazy as _

import daiquiri.core.env as env

CONFIG_DIR = Path(find_spec('config').origin).parent
BASE_DIR = CONFIG_DIR.parent
DAIQUIRI_APP = BASE_DIR.name.replace('-', '_')

BASE_URL = env.get_url('BASE_URL', '/')

DEBUG = env.get_bool('DEBUG')

SECRET_KEY = env.get('SECRET_KEY')

ALLOWED_HOSTS = env.get_list('ALLOWED_HOSTS', ['localhost', '127.0.0.1', '::1'])

if env.get_bool('PROXY'):
    USE_X_FORWARDED_HOST = True
    SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')

INTERNAL_IPS = env.get_list('INTERNAL_IPS', ['127.0.0.1'])

ADMINS = env.get_email_list('ADMINS')

SITE_ID = 1

ROOT_URLCONF = 'config.urls'

WSGI_APPLICATION = 'config.wsgi.application'

DATABASES = {
    'default': env.get_database('app'),
    'data': env.get_database('data'),
    'tap': env.get_database('data'),
    'oai': env.get_database('data')
}

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

if DATABASES['tap'].get('ENGINE') == 'django.db.backends.postgresql':
    DATABASES['tap']['OPTIONS'] = {
        'options': '-c search_path=%s' % env.get('TAP_SCHEMA', 'tap_schema')
    }
elif DATABASES['tap'].get('ENGINE') == 'django.db.backends.mysql':
    DATABASES['tap']['NAME'] = env.get('TAP_SCHEMA', 'tap_schema')

if DATABASES['oai'].get('ENGINE') == 'django.db.backends.postgresql':
    DATABASES['oai']['OPTIONS'] = {
        'options': '-c search_path=%s' % env.get('OAI_SCHEMA', 'oai_schema')
    }
elif DATABASES['oai'].get('ENGINE') == 'django.db.backends.mysql':
    DATABASES['oai']['NAME'] = env.get('OAI_SCHEMA', 'oai_schema')

ADAPTER_DATABASE = env.get_database_adapter()
ADAPTER_DOWNLOAD = env.get_download_adapter()

if DATABASES['data'].get('ENGINE') == 'django.db.backends.postgresql':
    USER_TABLESPACE = env.get('USER_TABLESPACE', 'pg_default')
else:
    USER_TABLESPACE = None

DATABASE_ROUTERS = [
    'daiquiri.core.routers.TapRouter',
    'daiquiri.core.routers.OaiRouter',
    'daiquiri.core.routers.DataRouter'
]

DJANGO_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
]

ADDITIONAL_APPS = [
    'rest_framework',
    'rest_framework.authtoken',
    'django_filters',
    'django_extensions',
    'vendor_files',
    'markdown',
    'compressor',
    'widget_tweaks',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'rules',
    'django_sendfile',
    'honeypot',
]

MIDDLEWARE = [
    'daiquiri.core.middleware.MultipleProxyMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.contrib.sites.middleware.CurrentSiteMiddleware'
]

TEMPLATES_DIR = BASE_DIR / 'templates/'
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [TEMPLATES_DIR] if TEMPLATES_DIR.exists() else [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django_settings_export.settings_export',
            ],
        },
    },
]

COMPRESS_PRECOMPILERS = (
    ('text/x-scss', 'django_libsass.SassCompiler'),
)

LANGUAGE_CODE = 'en-us'

TIME_ZONE = env.get('TIME_ZONE', 'UTC')

LOCALE_PATHS = (
    BASE_DIR / 'locale/',
)

LANGUAGES = (
    ('en', _('English')),
)

USE_I18N = True

USE_TZ = True

LOGIN_URL = BASE_URL + 'accounts/login/'
LOGIN_REDIRECT_URL = BASE_URL
LOGOUT_URL = BASE_URL + 'accounts/logout/'
LOGOUT_REDIRECT_URL = BASE_URL

CSRF_COOKIE_PATH = BASE_URL
LANGUAGE_COOKIE_PATH = BASE_URL
SESSION_COOKIE_PATH = BASE_URL

MEDIA_URL = BASE_URL + 'media/'
MEDIA_ROOT = BASE_DIR / 'media_root/'

STATIC_URL = BASE_URL + 'static/'
STATIC_ROOT = BASE_DIR / 'static_root/'

STATICFILES_DIR = BASE_DIR / 'static/'
STATICFILES_DIRS = [BASE_DIR / 'vendor/']
STATICFILES_DIRS += [STATICFILES_DIR] if STATICFILES_DIR.exists() else []

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder'
)

FIXTURE_DIRS = (
    BASE_DIR / 'fixtures',
)

AUTHENTICATION_BACKENDS = (
    'rules.permissions.ObjectPermissionBackend',
    'django.contrib.auth.backends.ModelBackend',
    'allauth.account.auth_backends.AuthenticationBackend',
)

ACCOUNT_LOGOUT_ON_GET = False
ACCOUNT_LOGOUT_REDIRECT_URL = BASE_URL
ACCOUNT_ADAPTER = 'daiquiri.auth.adapter.DaiquiriAccountAdapter'
ACCOUNT_SIGNUP_FORM_CLASS = 'daiquiri.auth.forms.SignupForm'
ACCOUNT_USER_DISPLAY = 'daiquiri.auth.utils.get_full_name'
ACCOUNT_USERNAME_VALIDATORS = 'daiquiri.auth.validators.username_validators'
ACCOUNT_USERNAME_MIN_LENGTH = 4
ACCOUNT_PASSWORD_MIN_LENGTH = 4
ACCOUNT_EMAIL_MAX_LENGTH = 190
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_EMAIL_CONFIRMATION_EXPIRE_DAYS = 7
# 'mandatory' or 'optional'
ACCOUNT_EMAIL_VERIFICATION = env.get('ACCOUNT_EMAIL_VERIFICATION', 'mandatory')
ACCOUNT_LOGIN_ON_EMAIL_CONFIRMATION = True

SOCIALACCOUNT_ADAPTER = 'daiquiri.auth.adapter.DaiquiriSocialAccountAdapter'
SOCIALACCOUNT_AUTO_SIGNUP = False

REST_FRAMEWORK = {
    'DEFAULT_THROTTLE_CLASSES': (
        'rest_framework.throttling.ScopedRateThrottle',
    ),
    'DEFAULT_THROTTLE_RATES': {
        'query.create': '10/second'
    }
}

SETTINGS_EXPORT = [
    'ACCOUNT_LOGOUT_ON_GET',
    'AUTH_WORKFLOW',
    'AUTH_TERMS_OF_USE',
    'LOGIN_URL',
    'LOGOUT_URL',
    'METADATA_COLUMN_PERMISSIONS',
    'QUERY_DROPDOWN_FUNCTIONS',
    'QUERY_DROPDOWNS',
    'QUERY_FORMS',
    'QUERY_PLOTS',
    'QUERY_DOWNLOADS',
    'QUERY_DOWNLOAD_FORMATS',
    'QUERY_UPLOAD',
    'SITE_PUBLISHER',
    'SITE_URL',
    'STATS_RESOURCE_TYPES'
]

DEFAULT_FROM_EMAIL = env.get('DEFAULT_FROM_EMAIL', 'info@example.com')
EMAIL_BACKEND = env.get('EMAIL_BACKEND', 'django.core.mail.backends.console.EmailBackend')
EMAIL_REPLY_TO = env.get('EMAIL_REPLY_TO')
EMAIL_HOST = env.get('EMAIL_HOST')
EMAIL_HOST_USER = env.get('EMAIL_HOST_USER')
EMAIL_HOST_PASSWORD = env.get('EMAIL_HOST_PASSWORD')
EMAIL_PORT = env.get('EMAIL_PORT', '25')
EMAIL_USE_TLS = env.get_bool('EMAIL_USE_TLS')

SENDFILE_BACKEND = env.get('SENDFILE_BACKEND', 'sendfile.backends.simple')

MEMCACHE_KEY_PREFIX = env.get('MEMCACHE_KEY_PREFIX')
if MEMCACHE_KEY_PREFIX:
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
            'LOCATION': '127.0.0.1:11211',
            'KEY_PREFIX': MEMCACHE_KEY_PREFIX
        }
    }

CELERY_BROKER_URL = env.get('CELERY_BROKER_URL', 'amqp://')
CELERY_BIN = env.get('CELERY_BIN', 'celery')
CELERY_PIDFILE_PATH = env.get_abspath('CELERY_PIDFILE_PATH')
CELERY_LOG_LEVEL = env.get('CELERY_LOG_LEVEL', 'INFO')
CELERY_LOG_PATH = env.get_abspath('CELERY_LOG_PATH')

HONEYPOT_ENABLED = True
HONEYPOT_FIELD_NAME = "Phone"
HONEYPOT_FIELD_VALUE = ""
HONEYPOT_FIELD_HIDDEN = True



from urllib.parse import urlparse

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    requires_system_checks = []
    can_import_settings = True

    def handle(self, *args, **options):

        parsed_url = urlparse(settings.CELERY_BROKER_URL)

        vhost = parsed_url.path.lstrip('/')

        print('rabbitmqctl add_user %s %s' % (parsed_url.username, parsed_url.password))
        print('rabbitmqctl add_vhost %s' % vhost)
        print('rabbitmqctl set_permissions -p %s %s ".*" ".*" ".*"' % (vhost, parsed_url.username))
        print('rabbitmqctl set_permissions -p %s admin ".*" ".*" ".*"' % vhost)



from django.urls import include, path
from rest_framework import routers

from .views import ExamplesView, JobsView, QueryView
from .viewsets import (DropdownViewSet, ExampleViewSet, FormViewSet,
                       PhaseViewSet, QueryJobViewSet, QueryLanguageViewSet,
                       QueueViewSet, StatusViewSet, DownloadViewSet)

app_name = 'query'

router = routers.DefaultRouter()
router.register(r'status', StatusViewSet, basename='status')
router.register(r'forms', FormViewSet, basename='form')
router.register(r'dropdowns', DropdownViewSet, basename='dropdown')
router.register(r'downloads', DownloadViewSet, basename='download')
router.register(r'jobs', QueryJobViewSet, basename='job')
router.register(r'examples', ExampleViewSet, basename='example')
router.register(r'queues', QueueViewSet, basename='queue')
router.register(r'querylanguages', QueryLanguageViewSet, basename='querylanguage')
router.register(r'phases', PhaseViewSet, basename='phase')

urlpatterns = [
    path(r'', QueryView.as_view(), name='query'),
    path(r'jobs/', JobsView.as_view(), name='jobs'),
    path(r'examples/', ExamplesView.as_view(), name='examples'),

    # rest api
    path(r'api/', include(router.urls)),
]



from .mysql import MySQLAdapter


class MariaDBAdapter(MySQLAdapter):

    def build_query(self, schema_name, table_name, query, timeout, max_records):
        # construct the actual query
        params = {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'query': query,
            'timeout': timeout,
            'max_records': max_records
        }

        if max_records is not None:
            return 'SET STATEMENT max_statement_time=%(timeout)s FOR CREATE TABLE %(schema)s.%(table)s ENGINE=ARIA ( %(query)s ) LIMIT %(max_records)s;' % params
        else:
            return 'SET STATEMENT max_statement_time=%(timeout)s FOR CREATE TABLE %(schema)s.%(table)s ENGINE=ARIA ( %(query)s );' % params

    def build_sync_query(self, query, timeout, max_records):
        # construct the actual query
        params = {
            'query': query,
            'timeout': timeout,
            'max_records': max_records
        }

        if max_records is not None:
            return 'SET STATEMENT max_statement_time=%(timeout)s FOR %(query)s LIMIT %(max_records)s;' % params
        else:
            return 'SET STATEMENT max_statement_time=%(timeout)s FOR %(query)s;' % params



STATS_RESOURCE_TYPES = [
    {
        'key': 'ARCHIVE_DOWNLOAD',
        'label': 'Archive downloads'
    },
    {
        'key': 'CONESEARCH',
        'label': 'Performed cone searches'
    },
    {
        'key': 'CUTOUT',
        'label': 'Performed cutouts'
    },
    {
        'key': 'FILE',
        'label': 'File downloads'
    },
    {
        'key': 'QUERY',
        'label': 'Queries'
    }
]



import os

from collections import OrderedDict

from django_sendfile import sendfile

from django.conf import settings
from django.http import Http404, FileResponse

from rest_framework import viewsets, mixins, filters
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError, NotFound
from rest_framework.decorators import action
from rest_framework.authentication import (
    SessionAuthentication,
    BasicAuthentication,
    TokenAuthentication
)

from daiquiri.core.viewsets import ChoicesViewSet, RowViewSetMixin
from daiquiri.core.permissions import HasModelPermission
from daiquiri.core.paginations import ListPagination
from daiquiri.core.utils import (
    get_client_ip,
    fix_for_json,
    filter_by_access_level,
    handle_file_upload,
    import_class
)
from daiquiri.jobs.viewsets import SyncJobViewSet, AsyncJobViewSet

from .models import QueryJob, DownloadJob, Example
from .serializers import (
    FormSerializer,
    DropdownSerializer,
    DownloadSerializer,
    QueryJobSerializer,
    QueryJobListSerializer,
    QueryJobRetrieveSerializer,
    QueryJobCreateSerializer,
    QueryJobUpdateSerializer,
    QueryJobUploadSerializer,
    QueryLanguageSerializer,
    ExampleSerializer,
    UserExampleSerializer,
    SyncQueryJobSerializer,
    AsyncQueryJobSerializer
)
from .permissions import HasPermission
from .utils import (
    get_download_config,
    get_format_config,
    get_quota,
    get_user_upload_directory,
    fetch_user_schema_metadata,
    handle_upload_param,
    ingest_uploads
)

from .filters import JobFilterBackend


class StatusViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    queryset = []

    def list(self, request):
        return Response([{
            'guest': not request.user.is_authenticated,
            'queued_jobs': None,
            'size': QueryJob.objects.get_size(request.user),
            'quota': get_quota(request.user),
            'upload_limit': get_quota(request.user, quota_settings='QUERY_UPLOAD_LIMIT')
        }])


class FormViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    serializer_class = FormSerializer

    def get_queryset(self):
        return settings.QUERY_FORMS


class DropdownViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    serializer_class = DropdownSerializer

    def get_queryset(self):
        return settings.QUERY_DROPDOWNS


class DownloadViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    serializer_class = DownloadSerializer

    def get_queryset(self):
        return settings.QUERY_DOWNLOADS


class QueryJobViewSet(RowViewSetMixin, viewsets.ModelViewSet):
    permission_classes = (HasPermission, )
    authentication_classes = (SessionAuthentication, BasicAuthentication, TokenAuthentication)
    pagination_class = ListPagination

    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter,
        JobFilterBackend
    )
    search_fields = ('id', 'run_id', 'table_name', 'phase')
    filter_fields = ('phase', )

    def get_queryset(self):
        queryset = QueryJob.objects.filter_by_owner(self.request.user).order_by('-creation_time')

        # hide TAP queries for the anonymous user
        if self.request.user.is_anonymous:
            queryset = queryset.filter(job_type=QueryJob.JOB_TYPE_INTERFACE)

        return queryset

    def get_serializer_class(self):
        if self.action == 'list':
            return QueryJobListSerializer
        elif self.action == 'retrieve' or self.action == 'abort':
            return QueryJobRetrieveSerializer
        elif self.action == 'create':
            return QueryJobCreateSerializer
        elif self.action == 'update' or self.action == 'partial_update':
            return QueryJobUpdateSerializer
        else:
            return QueryJobSerializer

    def get_throttles(self):
        if self.action == 'create':
            self.throttle_scope = 'query.create'

        return super(QueryJobViewSet, self).get_throttles()

    def perform_create(self, serializer):
        job = QueryJob(
            job_type=QueryJob.JOB_TYPE_INTERFACE,
            owner=(None if self.request.user.is_anonymous else self.request.user),
            run_id=serializer.data.get('run_id'),
            table_name=serializer.data.get('table_name'),
            query_language=serializer.data.get('query_language'),
            query=serializer.data.get('query'),
            queue=serializer.data.get('queue'),
            client_ip=get_client_ip(self.request)
        )
        job.process()
        job.save()
        job.run()

        # inject the job id into the serializers data
        serializer._data['id'] = job.id

    def perform_update(self, serializer):
        serializer.save()

    def perform_destroy(self, instance):
        instance.archive()

    @action(detail=False, methods=['get'])
    def tables(self, request):
        queryset = self.get_queryset().filter(phase=QueryJob.PHASE_COMPLETED)[:100]
        return Response(fetch_user_schema_metadata(request.user, queryset))

    @action(detail=False, methods=['post'], url_path='upload', url_name='upload')
    def upload(self, request):

        if not settings.QUERY_UPLOAD:
            raise Http404

        serializer = QueryJobUploadSerializer(data=request.data, context={
            'request': request,
            'view': self
        })
        serializer.is_valid(raise_exception=True)

        file_name = handle_file_upload(get_user_upload_directory(self.request.user), serializer.validated_data['file'])

        job = QueryJob(
            job_type=QueryJob.JOB_TYPE_INTERFACE,
            owner=(None if self.request.user.is_anonymous else self.request.user),
            run_id=serializer.validated_data.get('run_id'),
            table_name=serializer.validated_data.get('table_name'),
            client_ip=get_client_ip(self.request)
        )
        job.process(upload=True)
        job.save()
        job.ingest(file_name)

        serializer = QueryJobRetrieveSerializer(instance=job)
        return Response(serializer.data)

    @action(detail=True, methods=['put'])
    def abort(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
            job.abort()

            serializer = QueryJobRetrieveSerializer(instance=job)
            return Response(serializer.data)
        except QueryJob.DoesNotExist:
            raise Http404

    @action(detail=True, methods=['get'])
    def rows(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        # get the row query params from the request
        ordering, page, page_size, search, filters = self._get_query_params(job.columns())

        # get column names from the request
        column_names = self.request.GET.getlist('column')

        # get the count and the rows from the job
        count, results = job.rows(column_names, ordering, page, page_size, search, filters)

        # return ordered dict to be send as json
        return Response(OrderedDict((
            ('count', count),
            ('results', fix_for_json(results)),
            ('next', self._get_next_url(page, page_size, count)),
            ('previous', self._get_previous_url(page))
        )))

    @action(detail=True, methods=['get'])
    def columns(self, request, pk=None, format_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        return Response(job.columns())

    @action(detail=True, methods=['get'], url_name='download',
            url_path=r'download/(?P<download_key>[a-z\-]+)/(?P<download_job_id>[A-Za-z0-9\-]+)')
    def download(self, request, pk=None, download_key=None, download_job_id=None):
        try:
            self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        download_config = get_download_config(download_key)
        if download_config is None:
            raise ValidationError({'download': 'Download key "{}" is not supported.'.format(download_key)})

        download_job_model = import_class(download_config['model'])

        try:
            download_job = download_job_model.objects.get(query_job=pk, pk=download_job_id)
        except download_job_model.DoesNotExist:
            raise NotFound

        if download_job.phase == download_job.PHASE_COMPLETED and request.GET.get('download', True):
            return sendfile(request, download_job.file_path, attachment=True)
        else:
            return Response(download_job.phase)

    @action(detail=True, methods=['post'], url_name='create-download',
            url_path=r'download/(?P<download_key>[a-z\-]+)')
    def create_download(self, request, pk=None, download_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        download_config = get_download_config(download_key)
        if download_config is None:
            raise ValidationError({'download': 'Download key "{}" is not supported.'.format(download_key)})

        download_job_model = import_class(download_config['model'])

        params = {param: request.data.get(param) for param in download_config.get('params', [])}

        try:
            download_job = download_job_model.objects.get(query_job=job, **params)

            # check if the file was lost
            if download_job.phase == download_job.PHASE_COMPLETED and \
                    not os.path.isfile(download_job.file_path):

                # set the phase back to pending so that the file is recreated
                download_job.phase = download_job.PHASE_PENDING
                download_job.process()
                download_job.save()
                download_job.run()

        except download_job_model.DoesNotExist:
            download_job = download_job_model(
                job_type=QueryJob.JOB_TYPE_INTERFACE,
                owner=(None if self.request.user.is_anonymous else self.request.user),
                client_ip=get_client_ip(self.request),
                query_job=job,
                **params
            )
            download_job.process()
            download_job.save()
            download_job.run()

        return Response({
            'id': download_job.id
        })

    @action(detail=True, methods=['get'], url_path=r'stream/(?P<format_key>[A-Za-z0-9\-]+)', url_name='stream')
    def stream(self, request, pk=None, format_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist:
            raise NotFound

        try:
            format_config = get_format_config(format_key)
        except IndexError:
            raise ValidationError({'format': "Not supported."})

        try:
            download_job = DownloadJob.objects.get(query_job=job, format_key=format_key)

            # check if the file was lost
            if download_job.phase == download_job.PHASE_COMPLETED and os.path.isfile(download_job.file_path):
                # stream the previously created file
                return sendfile(request, download_job.file_path)
        except DownloadJob.DoesNotExist:
            pass

        # stream the table directly from the database
        response = FileResponse(job.stream(format_key), content_type=format_config['content_type'])
        return response


class ExampleViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    serializer_class = ExampleSerializer
    pagination_class = ListPagination
    queryset = Example.objects.all()

    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter
    )
    search_fields = ('name', 'description', 'query_string')

    @action(detail=False, methods=['get'], permission_classes=(HasPermission, ))
    def user(self, request):
        examples = Example.objects.filter_by_access_level(self.request.user)
        serializer = UserExampleSerializer(examples, many=True)
        return Response(serializer.data)


class QueueViewSet(ChoicesViewSet):
    permission_classes = (HasPermission, )

    def get_queryset(self):
        items = filter_by_access_level(self.request.user, settings.QUERY_QUEUES)
        return [(item['key'], item['label']) for item in items]


class QueryLanguageViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    serializer_class = QueryLanguageSerializer

    def get_queryset(self):
        return filter_by_access_level(self.request.user, settings.QUERY_LANGUAGES)


class PhaseViewSet(ChoicesViewSet):
    permission_classes = (HasPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)
    queryset = QueryJob.PHASE_CHOICES


class SyncQueryJobViewSet(SyncJobViewSet):
    permission_classes = (HasPermission, )
    serializer_class = SyncQueryJobSerializer

    parameter_map = {
        'FORMAT': 'response_format',
        'TABLE_NAME': 'table_name',
        'LANG': 'query_language',
        'QUERY': 'query'
    }

    def get_queryset(self):
        return QueryJob.objects.filter_by_owner(self.request.user)

    def handle_upload(self, job, upload_string):
        job.uploads = handle_upload_param(self.request, upload_string)
        ingest_uploads(job.uploads, job.owner)


class AsyncQueryJobViewSet(AsyncJobViewSet):
    permission_classes = (HasPermission, )
    serializer_class = AsyncQueryJobSerializer

    parameter_map = {
        'FORMAT': 'response_format',
        'TABLE_NAME': 'table_name',
        'LANG': 'query_language',
        'QUEUE': 'queue',
        'QUERY': 'query'
    }

    def get_queryset(self):
        return QueryJob.objects.filter_by_owner(self.request.user)

    def handle_upload(self, job, upload_string):
        job.uploads = handle_upload_param(self.request, upload_string)



import logging

from .base import BaseDownloadAdapter

logger = logging.getLogger(__name__)


class MysqldumpAdapter(BaseDownloadAdapter):

    def set_args(self, schema_name, table_name, data_only=False):
        self.args = ['mysqldump']
        if data_only:
            self.args += ['--compact', '--skip-extended-insert']

        if 'USER' in self.database_config and self.database_config['USER']:
            self.args.append('--user=%(USER)s' % self.database_config)

        if 'PASSWORD' in self.database_config and self.database_config['PASSWORD']:
            self.args.append('--password=%(PASSWORD)s' % self.database_config)

        if 'HOST' in self.database_config and self.database_config['HOST']:
            self.args.append('--host=%(HOST)s' % self.database_config)

        if 'PORT' in self.database_config and self.database_config['PORT']:
            self.args.append('--port=%(PORT)s' % self.database_config)

        self.args.append(schema_name)
        self.args.append(table_name)



import logging

from .base import BaseDownloadAdapter

logger = logging.getLogger(__name__)


class PgDumpAdapter(BaseDownloadAdapter):

    def set_args(self, schema_name, table_name, data_only=False):
        # command line for pg_dump:
        # pg_dump ... --dbname=postgresql://user:password@host:port/database --table=schema.table
        # pg_dump ... --user=user --host=host --port=port --dbname=database --table=schema.table

        self.args = ['pg_dump', '--no-owner']
        if data_only:
            self.args += ['--data-only', '--inserts']

        if 'PASSWORD' in self.database_config and self.database_config['PASSWORD']:
            if 'PORT' in self.database_config and self.database_config['PORT']:
                dbname = '--dbname=postgresql://%(USER)s:%(PASSWORD)s@%(HOST)s:%(PORT)s/%(NAME)s'
            else:
                dbname = '--dbname=postgresql://%(USER)s:%(PASSWORD)s@%(HOST)s/%(NAME)s'

            self.args.append(dbname % self.database_config)
        else:
            if 'USER' in self.database_config and self.database_config['USER']:
                self.args.append('--user=%(USER)s' % self.database_config)

            if 'HOST' in self.database_config and self.database_config['HOST']:
                self.args.append('--host=%(HOST)s' % self.database_config)

            if 'PORT' in self.database_config and self.database_config['PORT']:
                self.args.append('--port=%(PORT)d' % self.database_config)

            self.args.append('--dbname=%(NAME)s' % self.database_config)

        self.args.append('--table="%s"."%s"' % (schema_name, table_name))



# Generated by Django 3.2.8 on 2021-10-29 13:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_contact', '0005_django2'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contactmessage',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



from django import template
from re import IGNORECASE, compile, escape as rescape
from django.utils.safestring import mark_safe


register = template.Library()

@register.filter(name='highlight')
def highlight(text, search):
    rgx = compile(rescape(search), IGNORECASE)
    return mark_safe(
        rgx.sub(
            lambda m: '<b>{}</b>'.format(m.group()),
            text
        )
    )

@register.filter(name='underline')
def underline(text, search):
    rgx = compile(rescape(search), IGNORECASE)
    return mark_safe(
        rgx.sub(
            lambda m: '<u>{}</u>'.format(m.group()),
            text
        )
    )



import pytest
from django.urls import reverse

from ..models import Schema

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('test', 'test'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'admin': 200, 'manager': 200, 'user': 403, 'test': 403, 'anonymous': 403
    },
    'detail': {
        'admin': 200, 'manager': 200, 'user': 403, 'test': 403, 'anonymous': 403
    },
    'create': {
        'admin': 201, 'manager': 201, 'user': 403, 'test': 403, 'anonymous': 403
    },
    'update': {
        'admin': 200, 'manager': 200, 'user': 403, 'test': 403, 'anonymous': 403
    },
    'delete': {
        'admin': 204, 'manager': 204, 'user': 403, 'test': 403, 'anonymous': 403
    },
    'management': {
        'admin': 200, 'manager': 200, 'user': 403, 'test': 403, 'anonymous': 403
    },
    'user': {
        'admin': 200, 'manager': 200, 'user': 200, 'test': 200, 'anonymous': 200
    }
}

urlnames = {
    'list': 'metadata:schema-list',
    'detail': 'metadata:schema-detail',
    'management': 'metadata:schema-management',
    'user': 'metadata:schema-user'
}

instances = [3]


@pytest.mark.parametrize('username,password', users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()

    if response.status_code == 200:
        assert len(response.json()) == 5


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('pk', instances)
def test_detail(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Schema.objects.get(pk=pk)
    url = reverse(urlnames['detail'], args=[pk])
    response = client.get(url)
    assert response.status_code == status_map['detail'][username], response.json()

    if response.status_code == 200:
        assert response.json().get('name') == instance.name


@pytest.mark.parametrize('username,password', users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {
        'name': 'daiquiri_data_test',
        'discover': True
    })
    assert response.status_code == status_map['create'][username], response.json()


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('pk', instances)
def test_update(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Schema.objects.get(pk=pk)
    instance_name = instance.name
    url = reverse(urlnames['detail'], args=[pk])
    response = client.put(url, {
        'name': 'test',
    }, content_type='application/json')
    assert response.status_code == status_map['update'][username], response.json()

    instance.refresh_from_db()
    assert instance.name == ('test' if response.status_code == 200 else instance_name)


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('pk', instances)
def test_delete(db, client, username, password, pk):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.delete(url)
    assert response.status_code == status_map['delete'][username], response.json()
    assert Schema.objects.filter(pk=pk).exists() is not (response.status_code == 204)


@pytest.mark.parametrize('username,password', users)
def test_management(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['management'])
    response = client.get(url)
    assert response.status_code == status_map['management'][username], response.json()

    if response.status_code == 200:
        assert len(response.json()) == 5


@pytest.mark.parametrize('username,password', users)
def test_user(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['user'])
    response = client.get(url)
    assert response.status_code == status_map['user'][username], response.json()

    if username == 'admin':
        assert len(response.json()) == 4
    elif username == 'user':
        assert len(response.json()) == 3



from rest_framework import serializers

from daiquiri.jobs.serializers import SyncJobSerializer, AsyncJobSerializer

from .models import QueryJob, Example
from .validators import TableNameValidator, UploadFileValidator, UploadParamValidator


class FormSerializer(serializers.Serializer):

    key = serializers.CharField()
    form_service = serializers.SerializerMethodField()

    def get_form_service(self, obj):
        return obj['key'][0].upper() + obj['key'][1:] + 'FormService'


class DropdownSerializer(serializers.Serializer):

    key = serializers.CharField()
    dropdown_service = serializers.SerializerMethodField()
    options = serializers.SerializerMethodField()

    def get_dropdown_service(self, obj):
        return obj['key'][0].upper() + obj['key'][1:] + 'DropdownService'

    def get_options(self, obj):
        return obj['options']


class DownloadSerializer(serializers.Serializer):

    key = serializers.CharField()
    download_service = serializers.SerializerMethodField()
    options = serializers.SerializerMethodField()

    def get_download_service(self, obj):
        if obj.get('service'):
            return obj['key'][0].upper() + obj['key'][1:] + 'DownloadService'
        else:
            return None

    def get_options(self, obj):
        return obj.get('options', {})


class QueryJobSerializer(serializers.ModelSerializer):

    class Meta:
        model = QueryJob
        fields = (
            'id',
        )


class QueryJobListSerializer(serializers.ModelSerializer):

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'table_name',
            'creation_time',
            'run_id',
            'phase'
        )


class QueryJobRetrieveSerializer(serializers.ModelSerializer):

    sources = serializers.SerializerMethodField()
    columns = serializers.SerializerMethodField()

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'run_id',
            'phase',
            'creation_time',
            'start_time',
            'end_time',
            'execution_duration',
            'time_queue',
            'time_query',
            'destruction_time',
            'error_summary',
            'job_type',
            'schema_name',
            'table_name',
            'query_language',
            'query',
            'native_query',
            'actual_query',
            'queue',
            'nrows',
            'size',
            'sources',
            'columns'
        )

    def get_sources(self, obj):
        if obj.metadata:
            return obj.metadata.get('sources', [])
        else:
            return []

    def get_columns(self, obj):
        if obj.metadata:
            return obj.metadata.get('columns', [])
        else:
            return []


class QueryJobCreateSerializer(serializers.ModelSerializer):

    table_name = serializers.CharField(required=False, allow_blank=True, max_length=256,
                                       validators=[TableNameValidator()])
    queue = serializers.CharField(required=False)
    query_language = serializers.CharField(required=True)
    query = serializers.CharField(required=True)
    run_id = serializers.CharField(default='')

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'table_name',
            'queue',
            'query_language',
            'query',
            'run_id'
        )


class QueryJobUpdateSerializer(serializers.ModelSerializer):

    table_name = serializers.CharField(required=True, validators=[TableNameValidator()])

    class Meta:
        model = QueryJob
        fields = (
            'id',
            'table_name',
            'run_id'
        )


class QueryJobUploadSerializer(serializers.ModelSerializer):

    table_name = serializers.CharField(required=False, validators=[TableNameValidator()])
    run_id = serializers.CharField(default='')
    file = serializers.FileField(max_length=None, allow_empty_file=False, validators=[UploadFileValidator()])

    class Meta:
        model = QueryJob
        fields = (
            'table_name',
            'run_id',
            'file'
        )


class QueryLanguageSerializer(serializers.Serializer):

    id = serializers.SerializerMethodField(required=False)
    text = serializers.CharField(source='label')
    quote_char = serializers.CharField()

    def get_id(self, obj):
        return '%(key)s-%(version)s' % obj


class ExampleSerializer(serializers.ModelSerializer):

    class Meta:
        model = Example
        fields = (
            'id',
            'order',
            'name',
            'description',
            'query_language',
            'query_string',
            'access_level',
            'groups'
        )


class UserExampleSerializer(serializers.ModelSerializer):

    class Meta:
        model = Example
        fields = (
            'id',
            'order',
            'name',
            'description',
            'query_string',
            'query_language',
            'groups'
        )


class SyncQueryJobSerializer(SyncJobSerializer):

    RESPONSEFORMAT = serializers.CharField(required=False)
    FORMAT = serializers.CharField(required=False)

    TABLE_NAME = serializers.CharField(required=False, max_length=256, validators=[TableNameValidator()])
    LANG = serializers.CharField(required=True)
    QUERY = serializers.CharField(required=True)

    UPLOAD = serializers.CharField(required=False, default='', validators=[UploadParamValidator()])


class AsyncQueryJobSerializer(AsyncJobSerializer):

    RESPONSEFORMAT = serializers.CharField(required=False)
    FORMAT = serializers.CharField(required=False)

    TABLE_NAME = serializers.CharField(required=False, max_length=256, validators=[TableNameValidator()])
    QUEUE = serializers.CharField(required=False)
    LANG = serializers.CharField(required=True)
    QUERY = serializers.CharField(required=True)

    UPLOAD = serializers.CharField(required=False, default='', validators=[UploadParamValidator()])



import pytest
from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('user', 'user'),
    ('anonymous', None),
)


@pytest.mark.parametrize('username,password', users)
def test_home(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('home')
    response = client.get(url)
    assert response.status_code == 200



import pytest

from django.test import override_settings
from django.urls import reverse

from ..models import Example

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'detail': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'create': {
        'admin': 201, 'manager': 201, 'user': 403, 'anonymous': 403
    },
    'update': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'delete': {
        'admin': 204, 'manager': 204, 'user': 403, 'anonymous': 403
    },
    'user': {
        'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 403
    }
}

urlnames = {
    'list': 'query:example-list',
    'detail': 'query:example-detail',
    'user': 'query:example-user',
}

instances = [1]


@pytest.mark.parametrize('username,password', users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()

    if response.status_code == 200:
        assert response.json()['count'] == 4
        assert response.json()['results'][0]['access_level'] == 'PUBLIC'


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('pk', instances)
def test_detail(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Example.objects.get(pk=pk)
    url = reverse(urlnames['detail'], args=[pk])
    response = client.get(url)
    assert response.status_code == status_map['detail'][username], response.json()

    if response.status_code == 200:
        assert response.json().get('access_level') == instance.access_level


@pytest.mark.parametrize('username,password', users)
def test_create(db, client, mocker, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {
        'name': 'Test',
        'query_string': 'SELECT foo FROM bar',
        'access_level': 'PUBLIC',
        'query_language': 'adql-2.0'
    })
    assert response.status_code == status_map['create'][username], response.json()


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('pk', instances)
def test_update(db, client, mocker, username, password, pk):
    client.login(username=username, password=password)

    instance = Example.objects.get(pk=pk)
    instance_name = instance.name
    url = reverse(urlnames['detail'], args=[pk])
    response = client.put(url, {
        'name': 'Test',
        'query_string': 'SELECT foo FROM bar',
        'access_level': 'PUBLIC',
        'query_language': 'adql-2.0'
    }, content_type='application/json')
    assert response.status_code == status_map['update'][username], response.json()

    instance.refresh_from_db()
    assert instance.name == ('Test' if response.status_code == 200 else instance_name)


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('pk', instances)
def test_delete(db, client, mocker, username, password, pk):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.delete(url)
    assert response.status_code == status_map['delete'][username], response.json()
    assert Example.objects.filter(pk=pk).exists() is not (response.status_code == 204)


@pytest.mark.parametrize('username,password', users)
def test_user(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['user'])
    response = client.get(url)
    assert response.status_code == status_map['user'][username], response.json()

    if response.status_code == 200:
        assert len(response.json()) == (3 if username == 'admin' else 2)
        assert response.json()[0]['id'] == 1


@override_settings(QUERY_ANONYMOUS=True)
def test_user_anonymous(db, client):
    url = reverse(urlnames['user'])
    response = client.get(url)
    assert response.status_code == 200, response.json()

    if response.status_code == 200:
        assert len(response.json()) == 1
        assert response.json()[0]['id'] == 1



from django.db import models


class JobManager(models.Manager):

    def filter_by_owner(self, user):
        if not user or user.is_anonymous:
            return self.get_queryset().filter(owner=None)
        else:
            return self.get_queryset().filter(owner=user)

    def get_active(self, user):
        # get the number of PENDING, QUEUED or EXECUTING jobs
        return self.filter_by_owner(user).filter(phase__in=self.model.PHASE_ACTIVE)




from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import BadRequest
from django.utils.translation import gettext as _

from daiquiri.core.utils import get_detail_fields, sanitize_str

from .models import Profile


class HoneypotField(forms.CharField):
    hpstyle = 'border: 1px dashed'
    hplabel = settings.HONEYPOT_FIELD_NAME
    if settings.HONEYPOT_FIELD_HIDDEN is True:
        hpstyle = 'opacity: 0; position: absolute; top: 0; left: 0; height: 0; width: 0; z-index: -1;'
        hplabel = ''
    default_widget = forms.TextInput(
        {'style': hpstyle, 'tabindex': '-1', 'autocomplete': 'off'}
    )

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('widget', HoneypotField.default_widget)
        kwargs['required'] = False
        kwargs['label'] = self.hplabel
        super().__init__(*args, **kwargs)

    def validate(self, value):
        return value == settings.HONEYPOT_FIELD_VALUE

    def clean(self, value):
        cleaned_value = super().clean(value)
        if self.validate(cleaned_value) is True:
            return cleaned_value
        else:
            raise BadRequest('invalid request')

class UserForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ('first_name', 'last_name')
        widgets = {
            'first_name': forms.TextInput(attrs={'placeholder': _('First name')}),
            'last_name': forms.TextInput(attrs={'placeholder': _('Last name')}),
        }


class ProfileForm(forms.ModelForm):

    class Meta:
        model = Profile
        fields = ()

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)

        for key, field in get_detail_fields(settings.AUTH_DETAIL_KEYS):
            if self.instance.details and key in self.instance.details:
                field.initial = self.instance.details[key]

            self.fields[key] = field

    def save(self, *args, **kwargs):
        # create an empty details dict if it does not exist
        if not self.instance.details:
            self.instance.details = {}

        # store the form date for each detail key
        for detail_key in settings.AUTH_DETAIL_KEYS:
            self.instance.details[detail_key['key']] = self.cleaned_data[detail_key['key']]

        return super(ProfileForm, self).save(*args, **kwargs)


class SignupForm(ProfileForm):

    first_name = forms.CharField(max_length=30,
        label=_('First name'), widget=forms.TextInput(attrs={'placeholder': _('First name')}))
    last_name = forms.CharField(max_length=30,
        label=_('Last name'), widget=forms.TextInput(attrs={'placeholder': _('Last name')}))
    if settings.HONEYPOT_ENABLED is True:
        locals()[sanitize_str(settings.HONEYPOT_FIELD_NAME)] = HoneypotField()

    field_order = ['username', 'email', 'password1', 'password2']

    def __init__(self, *args, **kwargs):
        super(SignupForm, self).__init__(*args, **kwargs)

        # add a consent field, the label is added in the template
        if settings.AUTH_TERMS_OF_USE:
            self.fields['consent'] = forms.BooleanField(required=True)

    def signup(self, request, user):
        # create an empty details dict
        user.profile.details = {}

        # store the form date for each detail key
        for detail_key in settings.AUTH_DETAIL_KEYS:
            user.profile.details[detail_key['key']] = self.cleaned_data[detail_key['key']]

        # store the consent field
        if settings.AUTH_TERMS_OF_USE:
            user.profile.consent = self.cleaned_data['consent']

        # save the profile model
        user.profile.save()



import json

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.urls import reverse
from daiquiri.auth.models import Profile
from daiquiri.core.utils import get_referer_path_info, get_next
from django.views.generic import TemplateView
from django.http import Http404, HttpResponseRedirect, JsonResponse
from django.shortcuts import render
from django.urls import reverse_lazy
from rest_framework.authtoken.models import Token

from allauth.account.views import (
    logout as allauth_logout,
    PasswordChangeView as AllauthPasswordChangeView,
    PasswordSetView as AllauthPasswordSetView
)

from daiquiri.core.views import CSRFViewMixin, ModelPermissionMixin

from .forms import UserForm, ProfileForm


@login_required()
def profile_update(request):
    user_form = UserForm(request.POST or None, instance=request.user)
    profile_form = ProfileForm(request.POST or None, instance=request.user.profile)

    if request.method == 'POST':
        if 'cancel' in request.POST:
            return HttpResponseRedirect(get_next(request))

        if user_form.is_valid() and profile_form.is_valid():
            user_form.save()
            profile_form.save()
            return HttpResponseRedirect(get_next(request))

    return render(request, 'account/account_profile.html', {
        'user_form': user_form,
        'profile_form': profile_form,
        'next': get_referer_path_info(request, default='/')
    })


def terms_of_use(request):
    context = {}
    if settings.AUTH_TERMS_OF_USE == False:
        raise Http404 
    if request.user.is_authenticated:
        profile = Profile.objects.get(user=request.user)
        if request.method == 'POST':
            if 'consent' in request.POST:
                profile.consent = True
                profile.save()
        context["consent"] = profile.consent

    return render(request, 'account/terms_of_use_page.html', context)


@login_required()
def profile_json(request):
    return JsonResponse({
        'username': request.user.username
    })


@login_required()
def token(request):
    if request.method == 'POST':
        try:
            Token.objects.get(user=request.user).delete()
        except Token.DoesNotExist:
            pass

    token, created = Token.objects.get_or_create(user=request.user)
    return render(request, 'account/account_token.html', {
        'token': token
    })


def logout(request, *args, **kwargs):
    response = allauth_logout(request, *args, **kwargs)

    return response


class UsersView(ModelPermissionMixin, CSRFViewMixin, TemplateView):
    template_name = 'auth/users.html'
    permission_required = 'daiquiri_auth.view_profile'

    def get_context_data(self, **kwargs):
        # get urls to the admin interface to be used with angular
        user_admin_url = reverse('admin:auth_user_change', args=['row.id']).replace('row.id', '{$ row.id $}')
        profile_admin_url = reverse('admin:daiquiri_auth_profile_change', args=['row.id']).replace('row.id', '{$ row.id $}')

        detail_keys = settings.AUTH_DETAIL_KEYS
        for detail_key in detail_keys:
            detail_key['options_json'] = json.dumps(detail_key.get('options', {}))
            detail_key['model'] = 'service.current_row.details.%s' % detail_key['key']
            detail_key['errors'] = 'service.errors.%s' % detail_key['key']

        context = super(UsersView, self).get_context_data(**kwargs)
        context.update({
            'detail_keys': detail_keys,
            'user_admin_url': user_admin_url,
            'profile_admin_url': profile_admin_url
        })
        return context


class PasswordChangeView(AllauthPasswordChangeView):

    success_url = reverse_lazy('account_change_password_done')


password_change = login_required(PasswordChangeView.as_view())


class PasswordSetView(AllauthPasswordSetView):

    success_url = reverse_lazy('account_set_password_done')


password_set = login_required(PasswordSetView.as_view())



import daiquiri.core.env as env

VENDOR_CDN = env.get_bool('VENDOR_CDN')
VENDOR = {
    'jquery': {
        'url': 'https://code.jquery.com/',
        'js': [
            {
                'path': 'jquery-3.2.1.min.js',
                'sri': 'sha256-hwg4gsxgFZhOsEEamdOYGBf13FyQuiTwlAQgxVSNgt4=',
            }
        ]
    },
    'bootstrap': {
        'url': 'https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/',
        'js': [
            {
                'path': 'js/bootstrap.min.js',
                'sri': 'sha384-Tc5IQib027qvyjSMfHjOMaLkfuWVxZxUPnCJA7l2mCWNIpG9mGCD8wGNIcPD7Txa',
            }
        ],
        'css': [
            {
                'path': 'css/bootstrap.min.css',
                'sri': 'sha384-BVYiiSIFeK1dGmJRAkycuHAHRg32OmUcww7on3RYdg4Va+PmSTsz/K68vbdEjh4u',
            }
        ]
    },
    'font-awesome': {
        'url': 'https://maxcdn.bootstrapcdn.com/font-awesome/4.7.0/',
        'css': [
            {
                'path': 'css/font-awesome.min.css'
            }
        ],
        'font': [
            {
                'path': 'fonts/fontawesome-webfont.eot'
            },
            {
                'path': 'fonts/fontawesome-webfont.woff2'
            },
            {
                'path': 'fonts/fontawesome-webfont.woff'
            },
            {
                'path': 'fonts/fontawesome-webfont.ttf'
            },
            {
                'path': 'fonts/fontawesome-webfont.svg'
            }
        ]
    },
    'angular': {
        'url': 'https://ajax.googleapis.com/ajax/libs/angularjs/1.6.5/',
        'js': [
            {
                'path': 'angular.min.js'
            },
            {
                'path': 'angular-resource.min.js'
            },
            {
                'path': 'angular-sanitize.min.js'
            }
        ],
        'map': [
            {
                'path': 'angular.min.js.map'
            },
            {
                'path': 'angular-resource.min.js.map'
            },
            {
                'path': 'angular-sanitize.min.js.map'
            }
        ]
    },
    'ng-infinite-scroll': {
        'url': 'https://cdnjs.cloudflare.com/ajax/libs/ngInfiniteScroll/1.3.0/',
        'js': [
            {
                'path': 'ng-infinite-scroll.min.js',
                'sri': 'sha256-nUL1iPqMsX6n0f19hNGgkMsUgqQmP5k8PUWbDc1R/jU='
            }
        ]
    },
    'moment': {
        'url': 'https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.18.1/',
        'js': [
            {
                'path': 'moment.min.js',
                'sri': 'sha256-1hjUhpc44NwiNg8OwMu2QzJXhD8kcj+sJA3aCQZoUjg='
            }
        ]
    },
    'codemirror': {
        'url': 'https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.29.0/',
        'js': [
            {
                'path': 'codemirror.min.js',
                'sri': 'sha256-0LRLvWWVXwt0eH0/Bzd0PHICg/bSMDIe5sXgaDSpZaA='
            },
            {
                'path': 'addon/mode/overlay.min.js',
                'sri': 'sha256-ffWkw3Pn4ieMygm1vwdRKcMtBJ6E6kuBi8GlVVPXWEs='
            },
            {
                'path': 'mode/sql/sql.min.js',
                'sri': 'sha256-AYn1SMwJJCzQwlDkZLt7gAA3v8M14QZ7X5fGnJ2juYU='
            },
            {
                'path': 'mode/markdown/markdown.min.js',
                'sri': 'sha256-NzPBTZOCkVLaG/iTZe2XXF4cIyer9J0C0Z1FRN3WqNc='
            },
            {
                'path': 'addon/runmode/runmode.min.js',
                'sri': 'sha256-zO/lIsoS+q+kwzLO/YDjAGkeHUEw7Ql6ceZ4UMbityU='
            }
        ],
        'css': [
            {
                'path': 'codemirror.min.css',
                'sri': 'sha256-wluO/w4cnorJpS0JmcdTSYzwdb5E6u045qa4Ervfb1k='
            }
        ]
    },
    'Bokeh': {
        'url': 'https://cdnjs.cloudflare.com/ajax/libs/bokeh/2.4.3/',
        'js': [
            {
                'path': 'bokeh.min.js',
                'sri': 'sha512-PGHpUrgIu340bOgopWWl8jqvdwjQNPqdCQZAOBM1XUW7To2LCyOpPwMsdoMdTwIhhG8oaZx/UUfKpi4NMWaDFA=='
            },
            {
                'path': 'bokeh-gl.min.js',
                'sri': 'sha512-5d8x/ZAQ6s/wqnLTFSZGBLXjSXK6RFrOpvzpgIDuzzOs8tKYYSY2D6qlhSgTz/oW9vb0iJkjMYMA8GJJfALaTg=='
            },
            {
                'path': 'bokeh-api.min.js',
                'sri': 'sha512-M4sK7tJZoaDdYHfN4iRKfbD3W4P3hB9GGj3HnisgfQJCmrZUy4ajqjdxsMZjH+R7+tsN7+PsR4ouMiXYT2mJlA=='
            }
        ]
    }
}



from rest_framework import viewsets, filters, status
from rest_framework.response import Response
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import SessionAuthentication, TokenAuthentication

from django_filters.rest_framework import DjangoFilterBackend

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.viewsets import ChoicesViewSet
from daiquiri.core.permissions import HasModelPermission
from daiquiri.core.constants import ACCESS_LEVEL_CHOICES

from django.conf import settings

from .models import Schema, Table, Column, Function
from .serializers import (
    SchemaSerializer,
    TableSerializer,
    ColumnSerializer,
    FunctionSerializer
)
from .serializers.export import (
    SchemaSerializer as ExportSchemaSerializer,
    FunctionSerializer as ExportFunctionSerializer
)
from .serializers.management import (
    SchemaSerializer as ManagementSchemaSerializer,
    FunctionSerializer as ManagementFunctionSerializer
)
from .serializers.user import (
    SchemaSerializer as UserSchemaSerializer,
    FunctionSerializer as UserFunctionSerializer
)


class SchemaViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Schema.objects.all()
    serializer_class = SchemaSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    def create(self, request, *args, **kwargs):

        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        schema = serializer.save()

        if request.data.get('discover'):
            adapter = DatabaseAdapter()

            for table_metadata in adapter.fetch_tables(schema.name):
                table_metadata['schema'] = schema.id
                table_metadata['groups'] = [group.id for group in schema.groups.all()]
                for key in ['license', 'access_level', 'metadata_access_level']:
                    table_metadata[key] = getattr(schema, key)

                table_serializer = TableSerializer(data=table_metadata)
                if table_serializer.is_valid():
                    table = table_serializer.save()

                    for column_metadata in adapter.fetch_columns(schema.name, table.name):
                        column_metadata['table'] = table.id
                        column_metadata['groups'] = [group.id for group in table.groups.all()]
                        for key in ['access_level', 'metadata_access_level']:
                            column_metadata[key] = getattr(table, key)

                        column_serializer = ColumnSerializer(data=column_metadata)
                        if column_serializer.is_valid():
                            column_serializer.save()

        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=False)
    def management(self, request):
        queryset = Schema.objects.all()
        serializer = ManagementSchemaSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'], permission_classes=[])
    def user(self, request):
        # filter the schemas which are published for the groups of the user
        queryset = Schema.objects.filter_by_access_level(self.request.user)
        serializer = UserSchemaSerializer(queryset, context={'request': request}, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'], url_path='export', url_name='export-detail')
    def export_list(self, request):
        queryset = Schema.objects.all()
        serializer = ExportSchemaSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=True, methods=['get'], url_path='export', url_name='export-detail')
    def export_detail(self, request, pk=None):
        queryset = Schema.objects.get(pk=pk)
        serializer = ExportSchemaSerializer(queryset)
        return Response(serializer.data)


class TableViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Table.objects.all()
    serializer_class = TableSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    def create(self, request, *args, **kwargs):

        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        table = serializer.save()

        if request.data.get('discover'):
            adapter = DatabaseAdapter()

            for column_metadata in adapter.fetch_columns(table.schema.name, table.name):
                column_metadata['table'] = table.id
                column_metadata['groups'] = [group.id for group in table.groups.all()]
                for key in ['access_level', 'metadata_access_level']:
                    column_metadata[key] = getattr(table, key)

                column_serializer = ColumnSerializer(data=column_metadata)
                if column_serializer.is_valid():
                    column_serializer.save()

        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=False, methods=['get'])
    def discover(self, request):
        schema_name = request.GET.get('schema')
        table_name = request.GET.get('table')

        if schema_name and table_name:
            adapter = DatabaseAdapter()
            table_metadata = adapter.fetch_table(schema_name, table_name)
            table_metadata['nrows'] = adapter.fetch_nrows(schema_name, table_name)
            table_metadata['size'] = adapter.fetch_size(schema_name, table_name)
            return Response([table_metadata])
        else:
            return Response([])


class ColumnViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Column.objects.all()
    serializer_class = ColumnSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    @action(detail=False, methods=['get'])
    def discover(self, request):
        schema_name = request.GET.get('schema')
        table_name = request.GET.get('table')
        column_name = request.GET.get('column')

        if schema_name and table_name and column_name:
            return Response([DatabaseAdapter().fetch_column(schema_name, table_name, column_name)])
        else:
            return Response([])


class FunctionViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Function.objects.all()
    serializer_class = FunctionSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    @action(detail=False, methods=['get'])
    def management(self, request):
        queryset = Function.objects.all()
        serializer = ManagementFunctionSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'])
    def export(self, request):
        queryset = Function.objects.all()
        serializer = ExportFunctionSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'], permission_classes=[])
    def user(self, request):
        queryset = Function.objects.filter_by_access_level(self.request.user)
        serializer = UserFunctionSerializer(queryset, many=True)
        return Response(serializer.data)


class TableTypeViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Table.TYPE_CHOICES


class LicenseViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = settings.LICENSE_CHOICES


class AccessLevelViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = ACCESS_LEVEL_CHOICES



from django.conf import settings
from django.contrib.auth.models import Group
from django.shortcuts import get_object_or_404

from rest_framework import viewsets, mixins, filters
from rest_framework.decorators import action
from rest_framework.exceptions import MethodNotAllowed
from rest_framework.response import Response
from rest_framework.authentication import SessionAuthentication, TokenAuthentication

from daiquiri.core.permissions import HasModelPermission
from daiquiri.core.paginations import ListPagination

from .models import Profile
from .serializers import ProfileSerializer, GroupSerializer
from .permissions import IsAuthManager


class ProfileViewSet(mixins.UpdateModelMixin, mixins.ListModelMixin, mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Profile.objects.all()
    serializer_class = ProfileSerializer
    pagination_class = ListPagination

    filter_backends = (filters.SearchFilter, filters.OrderingFilter)
    ordering_fields = ('user__username', 'user__email', 'user__first_name', 'user__last_name')
    search_fields = ('user__username', 'user__email', 'user__first_name', 'user__last_name')

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def confirm(self, request, pk=None):
        if not settings.AUTH_WORKFLOW:
            raise MethodNotAllowed('put')

        profile = get_object_or_404(Profile, pk=pk)
        profile.confirm(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def reject(self, request, pk=None):
        if not settings.AUTH_WORKFLOW:
            raise MethodNotAllowed('put')

        profile = get_object_or_404(Profile, pk=pk)
        profile.reject(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def activate(self, request, pk=None):
        if not settings.AUTH_WORKFLOW:
            raise MethodNotAllowed('put')

        profile = get_object_or_404(Profile, pk=pk)
        profile.activate(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def disable(self, request, pk=None):
        profile = get_object_or_404(Profile, pk=pk)
        profile.disable(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def enable(self, request, pk=None):
        profile = get_object_or_404(Profile, pk=pk)
        profile.enable(request)
        return Response(self.get_serializer(profile).data)


class GroupViewSet(viewsets.ReadOnlyModelViewSet):
    permission_classes = (IsAuthManager, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Group.objects.order_by('name')
    serializer_class = GroupSerializer



from rest_framework import serializers

from .models import ContactMessage


class ContactMessageSerializer(serializers.ModelSerializer):

    status_label = serializers.SerializerMethodField()

    class Meta:
        model = ContactMessage
        fields = (
            'id',
            'author',
            'email',
            'subject',
            'status',
            'status_label',
            'created',
            'message',
            'user'
        )

    def get_status_label(self, obj):
        return dict(ContactMessage.STATUS_CHOICES)[obj.status]



__title__ = 'django-daiquiri'
__version__ = '0.4.5'
__author__ = 'Jochen Klar'
__email__ = 'mail@jochenklar.de'
__license__ = 'Apache-2.0'
__copyright__ = 'Copyright 2016-2023 Leibniz Institute for Astrophysics Potsdam (AIP)'

VERSION = __version__



import os
import logging

from django.conf import settings
from django.shortcuts import render
from django.utils.encoding import force_str
from django.utils.safestring import mark_safe
from django.utils.timezone import now

from django_sendfile import sendfile

from daiquiri.core.utils import get_client_ip, markdown
from daiquiri.stats.models import Record

from .models import Directory

logger = logging.getLogger(__name__)


def file_exists(file_path):
    absolute_file_path = os.path.join(settings.FILES_BASE_PATH, file_path)
    return os.path.isfile(absolute_file_path)


def get_file_path(file_path):
    if file_exists(file_path):
        return file_path
    elif not file_path or file_path.endswith('/'):
        # try different paths
        for path in [file_path.rstrip('/') + '.html',
                     file_path.rstrip('/') + '.md',
                     file_path + 'index.html',
                     file_path + 'index.md']:
            if file_exists(path):
                return path

    # return None if no file was found
    return None


def get_directory(user, file_path):
    # loop over all directories beginning with the hights depth and return as soon as a directory matches
    for directory in Directory.objects.order_by('-depth'):
        if os.path.normpath(file_path).startswith(directory.path):
            try:
                return Directory.objects.filter_by_access_level(user).get(pk=directory.pk)
            except Directory.DoesNotExist:
                return None


def check_file(user, file_path):
    return get_directory(user, file_path) is not None


def render_with_layout(request, file_path):

    context = {}
    absolute_file_path = os.path.join(settings.FILES_BASE_PATH, file_path)
    content = read_file_content(absolute_file_path)
    if content:
        context["content"] = content

    return render(request, 'files/layout.html', context)



def read_file_content(abs_file_path):
    """ Reads the content of a html- or md-file and returns html
    """
    if abs_file_path.endswith('.html') or abs_file_path.endswith('.md'):
        with open(abs_file_path) as f:
            file_content = f.read()

            if abs_file_path.endswith('.html'):
                return mark_safe(file_content)
            elif abs_file_path.endswith('.md'):
                return mark_safe(force_str(markdown(file_content)))
    else:
        return ""


def send_file(request, file_path, search=None):
    # create a stats record for this download
    resource = {
        'file_path': file_path
    }
    if search:
        resource['search'] = search

    Record.objects.create(
        time=now(),
        resource_type='FILE',
        resource=resource,
        client_ip=get_client_ip(request),
        user=request.user if request.user.is_authenticated else None
    )

    # send the file to the client
    absolute_file_path = os.path.join(settings.FILES_BASE_PATH, file_path)
    return sendfile(request, absolute_file_path)





from django.contrib import admin

from daiquiri.jobs.admin import JobAdmin

from .models import QueryJob, DownloadJob, QueryArchiveJob, Example


class QueryJobAdmin(JobAdmin):
    search_fields = JobAdmin.search_fields + ['schema_name', 'table_name']
    list_display = JobAdmin.list_display + ['schema_name', 'table_name', 'nrows']
    actions = ['abort_job', 'archive_job']


class DownloadJobAdmin(JobAdmin):
    search_fields = JobAdmin.search_fields + ['query_job__schema_name', 'query_job__table_name', 'format_key']
    list_display = JobAdmin.list_display + ['query_job', 'file_path']
    actions = ['abort_job', 'archive_job']


class QueryArchiveJobAdmin(JobAdmin):
    search_fields = JobAdmin.search_fields + ['query_job__schema_name', 'query_job__table_name', 'column_name', 'files']
    list_display = JobAdmin.list_display + ['query_job', 'file_path']
    actions = ['abort_job', 'archive_job']


class ExampleAdmin(admin.ModelAdmin):
    search_fields = ('name', 'query_string')
    list_display = ('order', 'name', 'query_string')
    list_display_links = ('name', )


admin.site.register(QueryJob, QueryJobAdmin)
admin.site.register(DownloadJob, DownloadJobAdmin)
admin.site.register(QueryArchiveJob, QueryArchiveJobAdmin)
admin.site.register(Example, ExampleAdmin)



from django.shortcuts import render
from django.http import Http404
from django.shortcuts import redirect

from daiquiri.metadata.utils import get_user_columns

from .utils import get_resolver


def table(request, schema_name, table_name):

    if get_user_columns(request.user, schema_name, table_name):
        return render(request, 'serve/table.html', {
            'schema': schema_name,
            'table': table_name
        })

    # if nothing worked, return 404
    raise Http404


def reference(request, key, value):

    resolver = get_resolver()
    if resolver is None:
        raise Http404()

    url = resolver.resolve(key, value)
    if url is None:
        raise Http404()

    return redirect(url)
